""" Refinement model
"""

from django.db import models
from django_extensions.db.fields import AutoSlugField

from core_main_app.commons import exceptions as exceptions


class Refinement(models.Model):
    name = models.CharField(max_length=50)
    xsd_name = models.CharField(max_length=50, default="")
    slug = AutoSlugField(max_length=50, overwrite=True, populate_from="name")
    # Cannot use a ReferenceField to template: not same database. Use the template hash instead.
    template_hash = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name

    @staticmethod
    def get_all():
        """Get all refinements.

        Returns: Refinement collection

        """
        return Refinement.objects.all()

    @staticmethod
    def get_all_filtered_by_template_hash(template_hash):
        """Get all refinements by template hash.

        Args:
            template_hash:

        Returns: Refinement collection

        """
        return Refinement.objects.all().filter(template_hash=template_hash)

    @staticmethod
    def create_and_save(name, xsd_name, template_hash):
        """Create and save a refinement.

        Args:
            name:
            xsd_name:
            template_hash:

        Returns:

        """
        return Refinement.objects.create(
            name=name, xsd_name=xsd_name, template_hash=template_hash
        )

    @staticmethod
    def check_refinements_already_exist_by_template_hash(template_hash):
        """Check if the refinements have already been generated for the template.

        Args:
            template_hash:

        Returns:
            Boolean: True/False

        """
        return (
            len(Refinement.get_all_filtered_by_template_hash(template_hash))
            > 0
        )

    @staticmethod
    def get_by_template_hash_and_by_slug(template_hash, slug):
        """Get refinement by template hash and by slug.

        Args:
            template_hash:
            slug:

        Returns: Refinement collection

        """
        try:
            return Refinement.objects.get(
                template_hash=template_hash, slug__startswith=slug
            )
        except Refinement.DoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def __str__(self):
        """Refinement as string

        Returns:

        """
        return self.name



""" Unit tests for `core_main_registry_app.views.admin.views` package
"""
from unittest.mock import patch, MagicMock

from django.test import SimpleTestCase

from core_main_registry_app.views.admin import views as admin_views


class TestUploadCustomResourceSaveCustomResources(SimpleTestCase):
    """Unit tests for `UploadCustomResource._save_custom_resources` method."""

    def setUp(self):
        self.view = admin_views.UploadCustomResource()
        self.mock_kwargs = {"request": MagicMock(), "template_id": MagicMock()}

    @patch.object(admin_views, "admin_render")
    @patch.object(admin_views, "template_api")
    def test_template_get_by_id_called(
        self, mock_template_api, mock_admin_render
    ):
        """test_template_get_by_id_called"""
        self.view._save_custom_resources(**self.mock_kwargs)

        mock_template_api.get_by_id.assert_called_with(
            self.mock_kwargs["template_id"],
            request=self.mock_kwargs["request"],
        )

    @patch.object(admin_views, "admin_render")
    @patch.object(admin_views, "template_api")
    def test_template_get_by_id_fails_puts_errors_in_context(
        self, mock_template_api, mock_admin_render
    ):
        """test_template_get_by_id_fails_puts_errors_in_context"""
        mock_template_api.get_by_id.side_effect = Exception(
            "mock_template_get_by_id_exception"
        )
        self.view._save_custom_resources(**self.mock_kwargs)

        self.assertIn(
            "errors",
            mock_admin_render.call_args_list[-1].kwargs["context"],
        )

    @patch.object(admin_views, "admin_render")
    @patch.object(admin_views, "load_json_string")
    @patch.object(admin_views, "template_api")
    def test_load_json_string_called(
        self, mock_template_api, mock_load_json_string, mock_admin_render
    ):
        """test_load_json_string_called"""
        mock_template_api.get_by_id.return_value = MagicMock()
        self.view._save_custom_resources(**self.mock_kwargs)

        mock_load_json_string.assert_called_with(
            self.mock_kwargs["request"]
            .FILES["json_file"]
            .read()
            .decode("utf-8")
        )

    @patch.object(admin_views, "admin_render")
    @patch.object(admin_views, "load_json_string")
    @patch.object(admin_views, "template_api")
    def test_load_json_string_fails_puts_errors_in_context(
        self, mock_template_api, mock_load_json_string, mock_admin_render
    ):
        """test_load_json_string_fails_puts_errors_in_context"""
        mock_template_api.get_by_id.return_value = MagicMock()
        mock_load_json_string.side_effect = Exception(
            "mock_load_json_string_exception"
        )
        self.view._save_custom_resources(**self.mock_kwargs)

        self.assertIn(
            "errors",
            mock_admin_render.call_args_list[-1].kwargs["context"],
        )

    @patch.object(admin_views, "admin_render")
    @patch.object(admin_views, "custom_resource_api")
    @patch.object(admin_views, "load_json_string")
    @patch.object(admin_views, "template_api")
    def test_replace_custom_resources_by_template_called(
        self,
        mock_template_api,
        mock_load_json_string,
        mock_custom_resource_api,
        mock_admin_render,
    ):
        """test_replace_custom_resources_by_template_called"""
        mock_template = MagicMock()
        mock_template_api.get_by_id.return_value = mock_template
        mock_data = MagicMock()
        mock_load_json_string.return_value = mock_data
        self.view._save_custom_resources(**self.mock_kwargs)

        mock_custom_resource_api.replace_custom_resources_by_template.assert_called_with(
            mock_template, mock_data
        )

    @patch.object(admin_views, "admin_render")
    @patch.object(admin_views, "custom_resource_api")
    @patch.object(admin_views, "load_json_string")
    @patch.object(admin_views, "template_api")
    def test_replace_custom_resources_by_template_puts_errors_in_context(
        self,
        mock_template_api,
        mock_load_json_string,
        mock_custom_resource_api,
        mock_admin_render,
    ):
        """test_replace_custom_resources_by_template_puts_errors_in_context"""
        mock_template_api.get_by_id.return_value = MagicMock()
        mock_load_json_string.return_value = MagicMock()
        mock_custom_resource_api.side_effect = Exception(
            "mock_custom_resource_api_exception"
        )
        self.view._save_custom_resources(**self.mock_kwargs)

        self.assertIn(
            "errors",
            mock_admin_render.call_args_list[-1].kwargs["context"],
        )

    @patch.object(admin_views, "HttpResponseRedirect")
    @patch.object(admin_views, "reverse")
    @patch.object(admin_views, "custom_resource_api")
    @patch.object(admin_views, "load_json_string")
    @patch.object(admin_views, "template_api")
    def test_success_returns_http_response_redirect(
        self,
        mock_template_api,
        mock_load_json_string,
        mock_custom_resource_api,
        mock_reverse,
        mock_http_response_redirect,
    ):
        """test_success_returns_http_response_redirect"""
        mock_template_api.get_by_id.return_value = MagicMock()
        mock_load_json_string.return_value = MagicMock()
        mock_custom_resource_api.return_value = None
        mock_http_response_redirect_value = MagicMock()
        mock_http_response_redirect.return_value = (
            mock_http_response_redirect_value
        )

        self.assertEqual(
            self.view._save_custom_resources(**self.mock_kwargs),
            mock_http_response_redirect_value,
        )



""" Category model
"""


from django.db import models
from django.core.exceptions import ObjectDoesNotExist
from django_extensions.db.fields import AutoSlugField
from mptt.models import MPTTModel, TreeForeignKey

from core_main_app.commons import exceptions as exceptions


class Category(MPTTModel):
    """Category object"""

    parent = TreeForeignKey(
        "self",
        null=True,
        on_delete=models.CASCADE,
        blank=True,
        related_name="children",
    )
    name = models.CharField(max_length=50)
    slug = AutoSlugField(max_length=50, overwrite=True, populate_from="name")
    path = models.CharField(max_length=255)
    value = models.CharField(max_length=255)
    refinement = models.ForeignKey("Refinement", on_delete=models.CASCADE)

    class Meta:
        """Meta"""

        verbose_name_plural = "Categories"

    class MPTTMeta(object):
        """MPTTMeta"""

        verbose_name_plural = "categories"
        unique_together = (("name", "slug", "parent"),)
        ordering = ("tree_id", "lft")

    @staticmethod
    def get_all_filtered_by_refinement_id(refinement_id):
        """Get all categories filtered by refinement id.

        Parameters:
                refinement_id:

        Returns: Category collection

        """
        return Category.objects.all().filter(refinement=refinement_id)

    @staticmethod
    def create_and_save(name, path, value, parent, refinement):
        """Create and save a category.

        Args:
            name:
            path:
            value:
            parent:
            refinement:

        Returns:

        """
        return Category.objects.create(
            name=name,
            path=path,
            value=value,
            parent=parent,
            refinement=refinement,
        )

    @staticmethod
    def get_by_id(category_id):
        """Get category by its id.

        Parameters:
            category_id:

        Returns:
            Category object

        """
        try:
            return Category.objects.get(pk=category_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_all_categories_by_parent_slug_and_refinement_id(
        parent_slug, refinement_id
    ):
        """Get all categories by parent_slug and refinement.

        Args:
            parent_slug:
            refinement_id:

        Returns:

        """
        try:
            return Category.objects.get(
                slug__startswith=parent_slug, refinement_id=refinement_id
            ).get_family()
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_all():
        """Get all categories.

        Returns: Category collection

        """
        return Category.objects.all()

    @staticmethod
    def get_all_categories_ids_from_name_and_refinement_id(
        name, refinement_id
    ):
        """Get all categories by name and refinement id.

        Args:
            name:
            refinement_id:

        Returns:

        """
        try:
            return Category.objects.get(
                name=name, refinement_id=refinement_id
            ).get_family()
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def __str__(self):
        """Category as string

        Returns:

        """
        return self.name



""" Core main registry app settings

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""
from os.path import join

from django.conf import settings

if not settings.configured:
    settings.configure()

REGISTRY_XSD_FILEPATH = getattr(
    settings,
    "REGISTRY_XSD_FILEPATH",
    join("core_main_registry_app", "xsd", "res-md.xsd"),
)
""" str: Registry xsd path used for the initialisation.
"""

REGISTRY_XSD_FILENAME = getattr(settings, "REGISTRY_XSD_FILENAME", "res-md")
""" str: Registry xsd filename used for the initialisation.
"""

CUSTOM_REGISTRY_FILE_PATH = getattr(
    settings,
    "CUSTOM_REGISTRY_FILE_PATH",
    join("core_main_registry_app", "json", "custom_registry.json"),
)
""" str: Custom registry configuration file path used for the initialisation.
"""

XSL_FOLDER_PATH = getattr(
    settings, "XSL_FOLDER_PATH", join("core_main_registry_app", "xsl")
)
""" str: Xsl folder path used for the initialisation.
"""

LIST_XSL_FILENAME = getattr(settings, "LIST_XSL_FILENAME", "registry-list.xsl")
"""" str : List xsl filename used for the initialisation.
"""

DETAIL_XSL_FILENAME = getattr(
    settings, "DETAIL_XSL_FILENAME", "registry-detail.xsl"
)
"""  str : Detail xsl filename used for the initialisation.
"""

ENABLE_BLOB_ENDPOINTS = getattr(settings, "ENABLE_BLOB_ENDPOINTS", False)
""" bool: Enable blob api and user views for blob management.
"""

ALLOW_MULTIPLE_SCHEMAS = getattr(settings, "ALLOW_MULTIPLE_SCHEMAS", False)
""" bool: Enable the use of multiple schemas in the registry.
"""



""" Menu utils
"""


def show_template_upload_menu(menu, allow_multiple_schemas=False):
    """Show template upload menu

    Args:
        menu:
        allow_multiple_schemas:

    Returns:

    """
    if not allow_multiple_schemas:
        for admin_menu_item in menu.items["admin"]:
            if admin_menu_item.title == "TEMPLATES":
                for template_menu_item_child in admin_menu_item.children:
                    if template_menu_item_child.title == "Upload New Template":
                        template_menu_item_child.visible = False



""" REST views for the registry Data API
"""
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.components.data import api as data_api
from core_main_app.rest.data.views import DataDetail
from core_main_app.utils.decorators import api_staff_member_required
from core_main_registry_app.components.data import api as registry_data_api


@api_view(["PATCH"])
def publish_data(request, pk):
    """Publish a data

    Args:

        request: HTTP request
        pk: ObjectId

    Returns:

        - code: 200
          content: Publication succeed
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    try:
        # Get data
        data_object = data_api.get_by_id(pk, request.user)

        # Publish
        registry_data_api.publish(data_object, request.user)

        # Return response
        return Response(status=status.HTTP_200_OK)
    except exceptions.DoesNotExist:
        content = {"message": "Data not found."}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {"message": str(ace)}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {"message": str(api_exception)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class DataDetailRegistry(DataDetail):
    """Retrieve or update a data"""

    @method_decorator(api_staff_member_required())
    def delete(self, request, pk):
        """Delete a Data

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 403
              content: Access forbidden
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        # Only superusers allowed to delete data
        if not request.user.is_superuser:
            content = {
                "message": "You must have superuser right to delete data."
            }
            return Response(content, status=status.HTTP_403_FORBIDDEN)

        return super(DataDetailRegistry, self).delete(request, pk)

    @method_decorator(api_staff_member_required())
    def patch(self, request, pk):
        """Data cannot be updated

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 403
              content: Access forbidden
        """
        # Only superusers allowed to update data
        if not request.user.is_superuser:
            content = {
                "message": "You must have superuser right to update data."
            }
            return Response(content, status=status.HTTP_403_FORBIDDEN)

        return super(DataDetailRegistry, self).patch(request, pk)



""" Unit tests for role extraction
"""
from unittest import TestCase

from core_main_registry_app.utils.role.extraction import role_extraction


class TestRoleExtraction(TestCase):
    """Tests for role extraction method."""

    def test_role_extraction_returns_role(self):
        """test_role_extraction_returns_role"""
        role = "role"
        result = role_extraction({"Resource": {"role": {"@xsi:type": role}}})
        self.assertEqual(result, [role])

    def test_role_extraction_returns_none_if_key_not_found(self):
        """test_role_extraction_returns_none_if_key_not_found"""
        role = "role"
        result = role_extraction(
            {"Resource": {"not_role": {"@xsi:type": role}}}
        )
        self.assertEqual(result, None)



""" Unit Test for Migrations
"""
from unittest.mock import patch

from django.db.migrations.exceptions import BadMigrationError
from django.test import TestCase

from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_registry_app import discover
from core_main_registry_app.components.custom_resource.models import (
    CustomResource,
)
from core_main_registry_app.components.refinement.models import Refinement


class TestInitData(TestCase):
    """TestInitData"""

    @patch.object(Refinement, "objects")
    @patch.object(CustomResource, "objects")
    @patch.object(TemplateVersionManager, "objects")
    @patch("core_main_registry_app.discover.main_registry_settings")
    def test_func_raises_error_if_REGISTRY_XSD_FILENAME_not_set(
        self,
        mock_settings,
        mock_tvm_objects,
        mock_cr_objects,
        mock_refinement_objects,
    ):
        """test_func_raises_error_if_REGISTRY_XSD_FILENAME_not_set"""
        mock_tvm_objects.count.return_value = 0
        mock_cr_objects.count.return_value = 0
        mock_refinement_objects.count.return_value = 0
        mock_settings.REGISTRY_XSD_FILENAME = None

        with self.assertRaises(BadMigrationError):
            discover.init_registry()

    @patch.object(Refinement, "objects")
    @patch.object(CustomResource, "objects")
    @patch.object(TemplateVersionManager, "objects")
    @patch("core_main_registry_app.discover.main_registry_settings")
    def test_func_raises_error_if_REGISTRY_XSD_FILEPATH_not_set(
        self,
        mock_settings,
        mock_tvm_objects,
        mock_cr_objects,
        mock_refinement_objects,
    ):
        """test_func_raises_error_if_REGISTRY_XSD_FILEPATH_not_set"""
        mock_tvm_objects.count.return_value = 0
        mock_cr_objects.count.return_value = 0
        mock_refinement_objects.count.return_value = 0
        mock_settings.REGISTRY_XSD_FILEPATH = None

        with self.assertRaises(BadMigrationError):
            discover.init_registry()

    @patch.object(Refinement, "objects")
    @patch.object(CustomResource, "objects")
    @patch.object(TemplateVersionManager, "objects")
    @patch("core_main_registry_app.discover.main_registry_settings")
    def test_func_raises_error_if_CUSTOM_REGISTRY_FILE_PATH_not_set(
        self,
        mock_settings,
        mock_tvm_objects,
        mock_cr_objects,
        mock_refinement_objects,
    ):
        """test_func_raises_error_if_CUSTOM_REGISTRY_FILE_PATH_not_set"""
        mock_tvm_objects.count.return_value = 0
        mock_cr_objects.count.return_value = 0
        mock_refinement_objects.count.return_value = 0
        mock_settings.CUSTOM_REGISTRY_FILE_PATH = None

        with self.assertRaises(BadMigrationError):
            discover.init_registry()

    @patch.object(Refinement, "objects")
    @patch.object(CustomResource, "objects")
    @patch.object(TemplateVersionManager, "objects")
    def test_func_raises_error_if_refinements_not_empty(
        self,
        mock_tvm_objects,
        mock_cr_objects,
        mock_refinement_objects,
    ):
        """test_func_raises_error_if_refinements_not_empty"""
        mock_tvm_objects.count.return_value = 0
        mock_cr_objects.count.return_value = 0
        mock_refinement_objects.count.return_value = 1

        self.assertIsNone(discover.init_registry())

    @patch.object(Refinement, "objects")
    @patch.object(CustomResource, "objects")
    @patch.object(TemplateVersionManager, "objects")
    def test_func_raises_error_if_custom_resources_not_empty(
        self,
        mock_tvm_objects,
        mock_cr_objects,
        mock_refinement_objects,
    ):
        """test_func_raises_error_if_custom_resources_not_empty"""
        mock_tvm_objects.count.return_value = 0
        mock_cr_objects.count.return_value = 1
        mock_refinement_objects.count.return_value = 0

        self.assertIsNone(discover.init_registry())

    @patch.object(Refinement, "objects")
    @patch.object(CustomResource, "objects")
    @patch.object(TemplateVersionManager, "objects")
    def test_func_raises_error_if_tvm_not_empty(
        self,
        mock_tvm_objects,
        mock_cr_objects,
        mock_refinement_objects,
    ):
        """test_func_raises_error_if_tvm_not_empty"""
        mock_tvm_objects.count.return_value = 1
        mock_cr_objects.count.return_value = 0
        mock_refinement_objects.count.return_value = 0

        self.assertIsNone(discover.init_registry())

    @patch.object(Refinement, "objects")
    @patch.object(CustomResource, "objects")
    @patch.object(TemplateVersionManager, "objects")
    @patch("core_main_registry_app.discover.main_registry_settings")
    def test_func_raises_error_if_loaded_does_not_exist(
        self,
        mock_settings,
        mock_tvm_objects,
        mock_cr_objects,
        mock_refinement_objects,
    ):
        """test_func_raises_error_if_loaded_does_not_exist"""
        mock_tvm_objects.count.return_value = 0
        mock_cr_objects.count.return_value = 0
        mock_refinement_objects.count.return_value = 0
        mock_settings.CUSTOM_REGISTRY_FILE_PATH = "bad"

        with self.assertRaises(BadMigrationError):
            discover.init_registry()



""" Custom admin site for the Custom Resource model
"""
from django.contrib import admin


class CustomResourceAdmin(admin.ModelAdmin):
    """CustomResourceAdmin"""

    readonly_fields = [
        "template",
        "name_in_schema",
        "title",
        "type",
        "role_choice",
        "role_type",
    ]
    exclude = ["slug"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Custom Resources"""
        return False



""" Template's registry api
"""

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template import api as template_api
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_registry_app.settings import REGISTRY_XSD_FILENAME


# NOTE: access control from main apis
def get_current_registry_template(request):
    """Get the current template used for the registry.

    Args:
        request:

    Returns:

    """
    try:
        template_version = template_version_manager_api.get_active_global_version_manager_by_title(
            REGISTRY_XSD_FILENAME, request=request
        )
        return template_api.get_by_id(
            template_version.current, request=request
        )
    except Exception as exception:
        raise exceptions.ModelError(str(exception))



""" Migrations
"""

# Generated by Django 3.2 on 2021-12-08 21:20

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import mptt.fields


class Migration(migrations.Migration):
    """Migration"""

    initial = True

    dependencies = [
        ("core_main_app", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Refinement",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50)),
                ("xsd_name", models.CharField(default="", max_length=50)),
                (
                    "slug",
                    django_extensions.db.fields.AutoSlugField(
                        blank=True,
                        editable=False,
                        overwrite=True,
                        populate_from="name",
                    ),
                ),
                ("template_hash", models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50)),
                (
                    "slug",
                    django_extensions.db.fields.AutoSlugField(
                        blank=True,
                        editable=False,
                        overwrite=True,
                        populate_from="name",
                    ),
                ),
                ("path", models.CharField(max_length=255)),
                ("value", models.CharField(max_length=255)),
                ("lft", models.PositiveIntegerField(editable=False)),
                ("rght", models.PositiveIntegerField(editable=False)),
                (
                    "tree_id",
                    models.PositiveIntegerField(db_index=True, editable=False),
                ),
                ("level", models.PositiveIntegerField(editable=False)),
                (
                    "parent",
                    mptt.fields.TreeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="children",
                        to="core_main_registry_app.category",
                    ),
                ),
                (
                    "refinement",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core_main_registry_app.refinement",
                    ),
                ),
            ],
            options={"verbose_name_plural": "Categories"},
        ),
        migrations.CreateModel(
            name="CustomResource",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name_in_schema",
                    models.CharField(blank=True, max_length=200),
                ),
                ("title", models.CharField(max_length=200)),
                ("slug", models.CharField(blank=True, max_length=200)),
                (
                    "description",
                    models.TextField(blank=True, default=None, null=True),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[("resource", "resource"), ("all", "all")],
                        max_length=200,
                    ),
                ),
                ("icon", models.CharField(max_length=200)),
                (
                    "icon_color",
                    models.CharField(
                        blank=True, default=None, max_length=200, null=True
                    ),
                ),
                (
                    "display_icon",
                    models.BooleanField(blank=True, default=None, null=True),
                ),
                (
                    "role_choice",
                    models.CharField(
                        blank=True, default=None, max_length=200, null=True
                    ),
                ),
                (
                    "role_type",
                    models.CharField(
                        blank=True, default=None, max_length=200, null=True
                    ),
                ),
                ("sort", models.PositiveIntegerField()),
                ("refinements", models.JSONField(blank=True, default=list)),
                (
                    "template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core_main_app.template",
                    ),
                ),
            ],
            options={
                "unique_together": {
                    ("sort", "template"),
                    ("title", "template"),
                },
            },
        ),
    ]



""" Integration Test for registry Template Version Manager Rest API
"""

from rest_framework import status

from core_main_app.rest.template_version_manager import views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.template_version_manager.fixtures.fixtures import (
    TemplateVersionManagerFixtures,
)

fixture_template = TemplateVersionManagerFixtures()


class TestCurrentTemplateVersion(IntegrationBaseTestCase):
    """Test Current Template Version"""

    fixture = fixture_template

    def test_patch_returns_http_200(self):
        """test_patch_returns_http_200"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_patch_wrong_id_returns_http_404(self):
        """test_patch_wrong_id_returns_http_404"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": -1},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_disabled_returns_http_400(self):
        """test_patch_disabled_returns_http_400"""

        # Arrange
        user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_2.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_global_as_user_returns_http_403(self):
        """test_patch_global_as_user_returns_http_403"""

        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_1_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_other_user_returns_http_403(self):
        """test_patch_other_user_returns_http_403"""

        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            views.CurrentTemplateVersion.as_view(),
            user,
            param={"pk": str(self.fixture.template_2_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



""" Unit tests for the fancytree widget
"""
from unittest import TestCase

from core_main_registry_app.utils.fancytree.widget import (
    FancyTreeWidget,
    FANCYTREE_CDN_PATH,
)
from tests.mocks import MockChoicesIterator


class TestFancyTreeWidgetRender(TestCase):
    """Tests for FancyTreeWidget render method."""

    def test_output_contains_fancytree_library(self):
        """test_output_contains_fancytree_library"""
        mock_fancytree_widget = FancyTreeWidget()
        mock_fancytree_widget.choices = MockChoicesIterator()
        results = mock_fancytree_widget.render(
            "mock_name", [1, 2, 3], {"id": "mock_refinement_id"}
        )

        self.assertIn(FANCYTREE_CDN_PATH, results)



""" Custom Resource model
"""
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.utils.text import slugify

from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template
from core_main_registry_app.constants import CUSTOM_RESOURCE_TYPE


class CustomResource(models.Model):
    """
    Custom resource object
    """

    template = models.ForeignKey(
        Template, blank=False, on_delete=models.CASCADE
    )
    name_in_schema = models.CharField(blank=True, max_length=200)
    title = models.CharField(blank=False, max_length=200)
    slug = models.CharField(blank=True, max_length=200)
    description = models.TextField(blank=True, default=None, null=True)
    type = models.CharField(
        blank=False,
        max_length=200,
        choices=[
            (
                CUSTOM_RESOURCE_TYPE.RESOURCE.value,
                CUSTOM_RESOURCE_TYPE.RESOURCE.value,
            ),
            (CUSTOM_RESOURCE_TYPE.ALL.value, CUSTOM_RESOURCE_TYPE.ALL.value),
        ],
    )
    icon = models.CharField(blank=False, max_length=200)
    icon_color = models.CharField(
        blank=True, default=None, null=True, max_length=200
    )
    display_icon = models.BooleanField(
        blank=True,
        default=None,
        null=True,
    )
    role_choice = models.CharField(
        blank=True, default=None, null=True, max_length=200
    )
    role_type = models.CharField(
        blank=True, default=None, null=True, max_length=200
    )
    sort = models.PositiveIntegerField()
    refinements = models.JSONField(blank=True, default=list)

    class Meta:
        """Meta"""

        unique_together = (("title", "template"), ("sort", "template"))

    def save(self, *args, **kwargs):
        """save.

        Args:

        Returns:

        """
        # set refinements to default value before cleaning if not set
        if not self.refinements:
            self.refinements = list()
        # manually clean object to trigger choices validation
        self.full_clean()
        if not self.slug:
            self.slug = slugify(self.title)
        return super().save(*args, **kwargs)

    @staticmethod
    def get_all_by_template(template):
        """Get all custom resource by template.

        Args:
            template:

        Returns:

        """
        return CustomResource.objects.filter(template=template).all()

    @staticmethod
    def get_custom_resource_by_template_and_type(template, type_name):
        """Get custom resource by template and type.

        Args:
            template:
            type_name:

        Returns:

        """
        return CustomResource.objects.filter(
            template=template, type=type_name
        ).all()

    @staticmethod
    def get_custom_resource_by_template_and_slug(template, slug):
        """Get custom resource by template and slug.

        Args:
            template:
            slug:

        Returns:

        """
        try:
            return CustomResource.objects.get(template=template, slug=slug)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_by_role_for_current_template(template, role):
        """Get custom resource by role for current template.

        Args:
            template:
            role:

        Returns:

        """
        try:
            list_custom_resources = CustomResource.objects.filter(
                template=template, role_choice=role
            ).all()
            if len(list_custom_resources) == 0:
                raise exceptions.ModelError(
                    "Can't find the custom resource with the given role: "
                    + role
                )
            return list_custom_resources[0]

        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def delete_custom_resources_by_template(template):
        """Delete all custom resources related to a template.

        Args:
            template:
        Returns:
        """
        CustomResource.objects.filter(template=template).delete()

    def __str__(self):
        """Custom Resource object as string

        Returns:

        """
        return str(self.title)



""" Unit tests for `rest.workspace.serializers` package.
"""
from unittest import TestCase
from unittest.mock import MagicMock, patch
from core_main_registry_app.rest.workspace import (
    serializers as workspace_serializers,
)


class TestWorkspaceSerializerCreate(TestCase):
    """Unit tests for `WorksapceSerializer.create` method."""

    def setUp(self):
        """setUp"""
        self.mock_serializer = workspace_serializers.WorkspaceSerializer()
        self.mock_kwargs = {"validated_data": MagicMock()}

    @patch.object(workspace_serializers, "workspace_api")
    def test_create_and_save_called(self, mock_workspace_api):
        """test_create_and_save_called"""
        self.mock_serializer.create(**self.mock_kwargs)

        mock_workspace_api.create_and_save.assert_called_with(
            title=self.mock_kwargs["validated_data"]["title"],
            owner_id=self.mock_kwargs["validated_data"]["user"].id,
            is_public=False,
        )

    @patch.object(workspace_serializers, "workspace_api")
    def test_is_public_in_validated_data_ignored(self, mock_workspace_api):
        """test_is_public_in_validated_data_ignored"""
        self.mock_kwargs["validated_data"]["is_public"] = True
        self.mock_serializer.create(**self.mock_kwargs)

        mock_workspace_api.create_and_save.assert_called_with(
            title=self.mock_kwargs["validated_data"]["title"],
            owner_id=self.mock_kwargs["validated_data"]["user"].id,
            is_public=False,
        )



""" Menu configuration for core_main_registry_app.
"""
from django.urls import reverse
from menu import Menu, MenuItem

from core_main_registry_app.settings import ALLOW_MULTIPLE_SCHEMAS
from core_main_registry_app.utils.menu.menu_utils import (
    show_template_upload_menu,
)

# Remove 'Upload New Template' option from menu
show_template_upload_menu(
    menu=Menu, allow_multiple_schemas=ALLOW_MULTIPLE_SCHEMAS
)

config_children = (
    MenuItem(
        "Configuration",
        reverse("core-admin:core_main_registry_app_custom_registry"),
        icon="upload",
    ),
)

Menu.add_item(
    "admin", MenuItem("CUSTOM REGISTRY", None, children=config_children)
)



""" Mocks for core_main_registry_app
"""
from unittest.mock import Mock


class MockField(Mock):
    label = "mock_label"


class MockChoicesIterator(Mock):
    def __init__(self):
        super().__init__()
        self.field = MockField()
        self._idx = 0
        self.iterator_list = []

    def __iter__(self):
        return self

    def __next__(self):
        if self._idx < len(self.iterator_list):
            self._idx += 1
            return self.iterator_list[self._idx - 1]
        else:
            raise StopIteration



"""Forms for admin views
"""
from django import forms


class UploadCustomResourcesForm(forms.Form):
    """
    Form to upload new Custom Resources
    """

    json_file = forms.FileField(
        label="Select a file",
        required=True,
        widget=forms.FileInput(attrs={"accept": ".json"}),
    )



""" Refinement API
"""

from core_main_registry_app.components.refinement.models import Refinement


def create_and_save(name, xsd_name, template_hash):
    """Create and save a refinement.

    Args:
        name:
        xsd_name:
        template_hash:

    Returns:

    """

    # Save refinement
    return Refinement.create_and_save(
        name=name, xsd_name=xsd_name, template_hash=template_hash
    )


def get_all():
    """Get all refinements.

    Returns: Refinement collection

    """
    return Refinement.get_all()


def get_all_filtered_by_template_hash(template_hash):
    """Get all refinements by template hash.

    Args:
        template_hash:

    Returns: Refinement collection

    """
    return Refinement.get_all_filtered_by_template_hash(
        template_hash=template_hash
    )


def check_refinements_already_exist_by_template_hash(template_hash):
    """Check if the refinements have already been generated for the template.

    Args:
        template_hash:

    Returns:
        Boolean: True/False

    """
    return Refinement.check_refinements_already_exist_by_template_hash(
        template_hash
    )


def get_by_template_hash_and_by_slug(template_hash, slug):
    """Get refinement by template hash and by slug.

    Args:
        template_hash:
        slug:

    Returns: Refinement collection

    """
    return Refinement.get_by_template_hash_and_by_slug(
        template_hash=template_hash, slug=slug
    )



"""
Tree representation of refinement.
"""

from collections import OrderedDict

from core_main_registry_app.constants import (
    UNSPECIFIED_CATEGORY,
    CATEGORY_SUFFIX,
    UNSPECIFIED_LABEL,
)


class TreeInfo(object):
    """
    Representation of refinement.
    """

    xsd_name = ""
    title = ""
    selected = False

    def __init__(self, xsd_name="", title="", path="", value=""):
        self.xsd_name = xsd_name
        self.title = title
        self.path = path
        self.value = value

    def __str__(self):
        return self.title

    def __eq__(self, other):
        return self.title == other.title

    def __hash__(self):
        return hash(self.title)

    def __lt__(self, other):
        return self.title < other.title

    def value_as_category(self):
        """value_as_category

        Returns

        """
        return f"{self.value}{CATEGORY_SUFFIX}"


def build_tree(tree, element_name, element_display_name, enums, dot_query):
    """Create a tree of refinements.

    Args:
        tree:
        element_name:
        element_display_name:
        enums:
        dot_query:

    Returns:

    """
    # Init tree.
    type_refinement = TreeInfo(
        xsd_name=element_name, title=element_display_name
    )
    first_node = tree.setdefault(type_refinement, OrderedDict())

    # For each enumerations, we create the tree representation.
    for enum in enums:
        parent_node = first_node
        levels = enum.attrib["value"].split(":")

        for i, level in enumerate(levels):
            # Create the tree info.
            graph = TreeInfo(
                xsd_name=level,
                title=level,
                path=dot_query,
                value=":".join(levels[: i + 1]),
            )
            g_node = parent_node.setdefault(graph, OrderedDict())
            parent_node = g_node

            # Case where it is the last element of the enum
            # check if we are in the unspecified case
            if len(levels) - 1 == i and _check_case_unspecified(
                enums, enum, i, level
            ):
                # Case unspecified: create a new node for the unspecified node
                title = (
                    f"{UNSPECIFIED_LABEL} {level}"
                    if UNSPECIFIED_CATEGORY
                    else level
                )
                graph = TreeInfo(
                    xsd_name=level,
                    title=title,
                    path=dot_query,
                    value=":".join(levels[: i + 1]),
                )
                parent_node.setdefault(graph, OrderedDict())

    return tree


def _check_case_unspecified(enums, current_enum, i, current_level):
    """Check if we can find the case of the unspecified.

    Args:
        enums: list of enums
        current_enum: current enum
        i: index of current level in current enum
        current_level: current level

    Returns:
    """
    for enum in enums:
        # We work only on enums different to the current enum
        if current_enum != enum:
            levels = enum.attrib["value"].split(":")
            # check if the current level is in the levels of the enum tested.
            # check if the current level is at the same position in the enum
            # check if the enum has more level
            if (
                current_level in levels
                and levels[i] == current_level
                and len(current_enum.attrib["value"])
                < len(enum.attrib["value"])
            ):
                return True
    return False



""" Fixtures files for Data
"""
from os.path import join, dirname, realpath

from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)
from core_main_app.components.workspace.models import Workspace


class DataRegistryFixtures(FixtureInterface):
    """Data fixtures"""

    data_1 = None
    data_2 = None
    workspace_1 = None
    workspace_2 = None
    template = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        mock_data_path = join(dirname(realpath(__file__)), "data")
        xml_path = join(mock_data_path, "data.xml")
        xml_without_status_path = join(
            mock_data_path, "data_without_status.xml"
        )

        with open(xml_path, "r", encoding="utf-8") as xml_file:
            xml_content_with_status = xml_file.read()
        with open(xml_without_status_path, "r", encoding="utf-8") as xml_file:
            xml_content_without_status = xml_file.read()

        self.data_1 = _create_data(self.template, xml_content_with_status)
        self.data_2 = _create_data(
            self.template, xml_content_without_status, self.workspace_2
        )
        self.data_collection = [self.data_1, self.data_2]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        mock_data_path = join(dirname(realpath(__file__)), "data")
        mock_xsl_path = join(mock_data_path, "res-md.xsd")
        with open(mock_xsl_path, "r", encoding="utf-8") as xsd_file:
            xsd = xsd_file.read()

        self.template = Template()
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "res-md.xsd"
        self.template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1",
            owner=None,
            read_perm_id="1",
            write_perm_id="1",
            is_public=True,
        )
        self.workspace_1.save()

        self.workspace_2 = Workspace(
            title="Workspace 2",
            owner="1",
            read_perm_id="1",
            write_perm_id="1",
            is_public=False,
        )
        self.workspace_2.save()


def _create_data(template, xml, workspace=None):
    data = Data(
        template=template,
        user_id="1",
        dict_content=None,
        title="title",
        workspace=workspace,
    )
    data.xml_content = xml
    data.convert_to_dict()
    data.save()
    return data



""" Authentication tests for Template Version Manager REST API
"""
from django.test import SimpleTestCase
from unittest.mock import patch
from rest_framework import status

from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_main_registry_app.rest.template_version_manager import (
    views as template_version_manager_views,
)


class TestCurrentTemplateVersionPatchPermission(SimpleTestCase):
    """Test Current Template Version Patch Permission"""

    def setUp(self):
        """setUp"""

        self.fake_id = -1

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_registry_app.components.version_manager.api.set_current")
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(
        self,
        template_get_by_id,
        version_manager_set_current,
    ):
        """test_staff_returns_http_200"""

        version_manager_set_current.return_value = {}
        template_get_by_id.return_value = Template(user=None)

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



# Generated by Django 4.2.8 on 2024-03-13 21:06

from django.db import migrations


def forwards_func(apps, schema_editor):
    """Initialize the registry.
       Add the registry template, refinements, and resources.

    Returns:

    """
    from core_main_registry_app.discover import init_registry

    # Initialize registry
    init_registry()


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_registry_app", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(forwards_func, migrations.RunPython.noop)
    ]



""" Apps file for setting core main registry app package when app is ready
"""
import sys

from django.apps import AppConfig


class InitApp(AppConfig):
    """Core main registry app application settings"""

    name = "core_main_registry_app"
    verbose_name = "Core Main Registry App"

    def ready(self):
        """Run when the app is ready.

        Returns:

        """
        if "migrate" not in sys.argv:
            from core_main_registry_app.utils.refinement import (
                watch as refinement_watch,
            )

            # Init registry
            refinement_watch.init()



""" Unit Test for Refinement API
"""

from django.test import TestCase
from core_main_app.commons.exceptions import DoesNotExist
from core_main_registry_app.components.refinement import (
    api as refinement_api,
)
from core_main_registry_app.components.refinement.models import (
    Refinement,
)


class TestRefinementCreateAndSave(TestCase):
    """
    Test Refinement Create And Save
    """

    def test_create_and_save_refinement_creates_refinement(self):
        """test_create_and_save_refinement_creates_refinement"""
        # Act
        result = refinement_api.create_and_save("Refinement", "", "")
        # Assert
        self.assertIsInstance(result, Refinement)
        self.assertEqual(result.name, "Refinement")


class TestRefinementGetAll(TestCase):
    """
    Test Refinement Get All
    """

    def test_refinement_get_all_refinements_returns_list_of_refinements(self):
        """test_refinement_get_all_refinements_returns_list_of_refinements"""
        # Act
        result = refinement_api.get_all()
        # Assert

        for obj in list(result):
            self.assertTrue(isinstance(obj, Refinement))

    def test_refinement_get_all_refinement_returns_list(self):
        """test_get_all_refinements_returns_empty_list"""
        # Arrange
        refinement_count = Refinement.objects.count()
        refinement_1 = create_refinement()
        refinement_2 = create_refinement()
        # Act
        result = refinement_api.get_all()
        # Assert

        self.assertEqual(len(result), refinement_count + 2)
        self.assertTrue(refinement_1 in result)
        self.assertTrue(refinement_2 in result)


class TestRefinementGetAllFilteredByTemplateHash(TestCase):
    """
    Test Refinement Get All Filtered By Template Hash
    """

    def test_refinement_get_all_filtered_by_template_hash_returns_empty_list(
        self,
    ):
        """test_refinement_get_all_filtered_by_template_hash_returns_empty_list"""
        # Arrange
        create_refinement()
        # Act
        result = refinement_api.get_all_filtered_by_template_hash("hash")
        # Assert

        self.assertEqual(len(result), 0)

    def test_refinement_get_all_filtered_by_template_hash_returns_list(self):
        """test_refinement_get_all_filtered_by_template_hash_returns_list"""
        # Arrange
        refinement_1 = create_refinement()
        refinement_2 = create_refinement()
        # Act
        result = refinement_api.get_all_filtered_by_template_hash("")
        # Assert

        self.assertEqual(len(result), 2)
        self.assertTrue(refinement_1 in result)
        self.assertTrue(refinement_2 in result)


class TestRefinementCheckRefinementsAlreadyExistByTemplateHash(TestCase):
    """
    Test Refinement Check Refinements Already Exist By Template Hash
    """

    def test_check_refinements_already_exist_by_template_hash_returns_false(
        self,
    ):
        """test_check_refinements_already_exist_by_template_hash_returns_false"""
        # Arrange
        create_refinement()
        # Act
        result = (
            refinement_api.check_refinements_already_exist_by_template_hash(
                "hash"
            )
        )
        # Assert
        self.assertFalse(result)

    def test_check_refinements_already_exist_by_template_hash_returns_true(
        self,
    ):
        """test_check_refinements_already_exist_by_template_hash_returns_true"""
        # Arrange
        create_refinement()

        # Act
        result = refinement_api.get_all_filtered_by_template_hash("")
        # Assert
        self.assertTrue(result)


class TestRefinementGetByTemplateHashAndBySlug(TestCase):
    """
    Test Refinement Get By Template Hash And By Slug
    """

    def test_refinement_get_by_template_hash_and_by_slug_raises_does_not_exist_exception(
        self,
    ):
        """test_get_by_template_hash_and_by_slug_raises_error_does_not_exist"""
        # Arrange
        create_refinement()
        # Act # Assert
        with self.assertRaises(DoesNotExist):
            refinement_api.get_by_template_hash_and_by_slug("hash", "slug")

    def test_refinement_get_by_template_hash_and_by_slug_returns_refinement(
        self,
    ):
        """test_refinement_get_by_template_hash_and_by_slug_returns_refinement"""
        # Arrange
        create_refinement()

        # Act
        result = refinement_api.get_by_template_hash_and_by_slug("", "")
        # Assert
        self.assertIsInstance(result, Refinement)


def create_refinement():
    """create_refinement

    Return:
    """
    return Refinement.create_and_save("Refinement", "", "")



""" Url router for the core main registry app
"""
from django.conf.urls import include
from django.contrib.auth.decorators import login_required
from django.urls import re_path
from drf_spectacular.views import SpectacularAPIView, SpectacularSwaggerView

from core_main_app.components.data import api as data_api
from core_main_app.utils.rendering import render
from core_main_app.utils.urls import get_auth_urls
from core_main_app.views.common import (
    ajax as common_ajax,
    views as common_views,
)
from core_main_app.views.user import views as user_views, ajax as user_ajax
from core_main_registry_app.settings import ENABLE_BLOB_ENDPOINTS

# FIXME: Check needed URLs for registry and remove the others.

urlpatterns = [
    re_path(r"^$", user_views.homepage, name="core_main_app_homepage"),
    re_path("api/schema/", SpectacularAPIView.as_view(), name="schema"),
    re_path(
        "docs/api/",
        SpectacularSwaggerView.as_view(url_name="schema"),
        name="swagger_view",
    ),
    re_path(
        r"^locked",
        common_views.defender_error_page,
        name="core_main_app_locked",
    ),
    re_path(r"^rest/", include("core_main_registry_app.rest.urls")),
    re_path(
        r"^data",
        common_views.ViewData.as_view(),
        name="core_main_app_data_detail",
    ),
    re_path(
        r"^template/versions/(?P<version_manager_id>\w+)",
        user_views.manage_template_versions,
        name="core_main_app_manage_template_versions",
    ),
    re_path(
        r"^template/xslt/(?P<template_id>\w+)",
        login_required(
            common_views.TemplateXSLRenderingView.as_view(
                rendering=render,
                template_name="core_main_app/common/templates_xslt/main.html",
                save_redirect="core_main_app_manage_template_versions",
            )
        ),
        name="core_main_app_template_xslt",
    ),
    re_path(
        r"^edit-rights/(?P<workspace_id>\w+)$",
        login_required(common_views.EditWorkspaceRights.as_view()),
        name="core_main_edit_rights_workspace",
    ),
    re_path(
        r"^change-workspace",
        login_required(
            user_ajax.LoadFormChangeWorkspace.as_view(
                show_global_workspace=False
            )
        ),
        name="core_main_change_workspace",
    ),
    re_path(
        r"^assign-data-workspace",
        login_required(user_ajax.AssignView.as_view(api=data_api)),
        name="core_main_assign_data_workspace",
    ),
    re_path(
        r"^assign-workspace",
        login_required(user_ajax.AssignView.as_view(api=data_api)),
        name="core_main_assign_workspace",
    ),
    re_path(
        r"^add-user-form",
        user_ajax.load_add_user_form,
        name="core_main_edit_rights_users_form",
    ),
    re_path(
        r"^add-user-right-to-workspace",
        user_ajax.add_user_right_to_workspace,
        name="core_main_add_user_right_to_workspace",
    ),
    re_path(
        r"^switch-right", user_ajax.switch_right, name="core_main_switch_right"
    ),
    re_path(
        r"^remove-rights",
        user_ajax.remove_user_or_group_rights,
        name="core_main_remove_rights",
    ),
    re_path(
        r"^add-group-form",
        user_ajax.load_add_group_form,
        name="core_main_edit_rights_groups_form",
    ),
    re_path(
        r"^add-group-right-to-workspace",
        user_ajax.add_group_right_to_workspace,
        name="core_main_add_group_right_to_workspace",
    ),
    re_path(
        r"^change-data-display",
        user_ajax.change_data_display,
        name="core_main_add_change_data_display",
    ),
    re_path(
        r"^xml-editor/data",
        login_required(
            common_views.DataXMLEditor.as_view(),
        ),
        name="core_main_app_xml_text_editor_view",
    ),
    re_path(
        r"^json-editor/data",
        login_required(
            common_views.DataJSONEditor.as_view(),
        ),
        name="core_main_app_json_text_editor_view",
    ),
    re_path(
        r"^set-timezone",
        user_views.set_timezone,
        name="core_main_set_timezone",
    ),
    re_path(
        r"^template/(?P<pk>[\w-]+)/edit/$",
        common_ajax.EditTemplateVersionManagerView.as_view(),
        name="core_main_app_edit_template",
    ),
]

urlpatterns.extend(get_auth_urls())

if ENABLE_BLOB_ENDPOINTS:
    from core_main_app.components.blob import api as blob_api

    urlpatterns.append(
        re_path(
            r"^blob/(?P<pk>[\w-]+)/metadata",
            login_required(common_views.ManageBlobMetadata.as_view()),
            name="core_main_app_blob_metadata",
        )
    )
    urlpatterns.append(
        re_path(
            r"^blob",
            common_views.ViewBlob.as_view(),
            name="core_main_app_blob_detail",
        )
    )
    urlpatterns.append(
        re_path(
            r"^assign-blob-workspace",
            user_ajax.AssignView.as_view(api=blob_api),
            name="core_main_assign_blob_workspace",
        )
    )
    urlpatterns.append(
        re_path(
            r"^add-metadata-form",
            user_ajax.LoadBlobMetadataForm.as_view(),
            name="core_main_blob_metadata_form",
        )
    )
    urlpatterns.append(
        re_path(
            r"^add-metadata-to-blob",
            user_ajax.AddMetadataToBlob.as_view(),
            name="core_main_blob_add_metadata",
        )
    )
    urlpatterns.append(
        re_path(
            r"^remove-metadata-from-blob",
            user_ajax.RemoveMetadataFromBlob.as_view(),
            name="core_main_blob_remove_metadata",
        )
    )
    urlpatterns.append(
        re_path(
            r"^upload-file",
            user_ajax.UploadFile.as_view(),
            name="core_main_upload_file",
        )
    )



""" Registry tasks
"""
from celery import shared_task

from core_main_app.system import api as system_api
from core_main_registry_app.utils.refinement.refinement import init_refinements


@shared_task
def init_refinement_task(template_id):
    """Asynchronous tasks init refinement

    Args:
        template_id:

    Returns:

    """
    template = system_api.get_template_by_id(template_id)
    init_refinements(template)



""" Unit tests for menus
"""
from unittest import TestCase
from unittest.mock import MagicMock

from core_main_registry_app.utils.menu.menu_utils import (
    show_template_upload_menu,
)


class TestMenus(TestCase):
    """Tests menus"""

    def setUp(self) -> None:
        """

        Returns:

        """
        self.mock_menu_item_child = MagicMock(
            title="Upload New Template", visible=True
        )
        self.mock_menu_item = MagicMock(
            title="TEMPLATES", children=[self.mock_menu_item_child]
        )
        self.mock_menu = MagicMock(items={"admin": [self.mock_menu_item]})

    def test_template_upload_in_menus(self):
        """test_template_upload_in_menus"""

        show_template_upload_menu(self.mock_menu, allow_multiple_schemas=True)
        self.assertTrue(self.mock_menu_item_child.visible)

    def test_template_upload_not_in_menus(self):
        """test_template_upload_not_in_menus"""

        show_template_upload_menu(self.mock_menu, allow_multiple_schemas=False)
        self.assertFalse(self.mock_menu_item_child.visible)



"""
Mongo query creation for the refinements.
"""

import logging
import operator
from functools import reduce

from django.db.models import Q

from core_main_app.commons import exceptions as exceptions
from core_main_registry_app.commons.constants import DataStatus
from core_main_registry_app.components.category import api as category_api
from core_main_registry_app.components.refinement import api as refinement_api
from core_main_registry_app.components.template import (
    api as template_registry_api,
)
from core_main_registry_app.constants import PATH_STATUS

logger = logging.getLogger(
    "core_main_registry_app.utils.refinement.mongo_query"
)


def build_refinements_query(refinements):
    """Build the refinements query.

    Args:
        refinements:

    Returns:

    """
    or_queries = []
    and_query = {}

    try:
        # transform the refinement in mongo query
        for refinement in refinements:
            queries = dict()
            in_queries = {}
            # For each category in the refinement
            for category_id in refinement:
                try:
                    # Get category
                    category = category_api.get_by_id(category_id)
                    dot_notation = category.path
                    value = category.value
                    # If dot notation already exists, append to the dict
                    if dot_notation in queries:
                        queries[dot_notation].append(value)
                    # Create a dict with the dot notation as the key
                    else:
                        queries[dot_notation] = [value]
                except (exceptions.DoesNotExist, Exception) as exception:
                    logger.warning(
                        "Impossible to find the category (%s): %s.",
                        str(len(category_id)),
                        str(exception),
                    )

            for query in queries:
                # Create the query with $in
                key = query
                values = {"$in": queries[query]}
                in_queries[key] = values
                # Case of the element has attributes
                in_queries[key + ".#text"] = values

            if len(in_queries) > 0:
                # $or between categories belonging to the same refinement
                or_queries.append(
                    {"$or": [{x: in_queries[x]} for x in in_queries]}
                )

        if len(or_queries) > 0:
            # $and between refinements
            and_query = {"$and": or_queries}

        return and_query
    except Exception as exception:
        logger.error(
            "Something went wrong during the creation of the refinement query. Search "
            "won't be refined: %s.",
            str(exception),
        )
        return {}


def get_refinement_selected_values_from_query(query, request):
    """get the refinement selected values from a json query

    Args:
        query:
        request:

    Returns:
        {
            refinement_name: [cat_id, cat_id ,cat_id],
            refinement_name: [cat_id, cat_id ,cat_id],
            refinement_name: [cat_id, cat_id ,cat_id],
        }

    """
    # create a list of key (category), value_list (selected values)
    category_values_list = {}
    return_value = {}

    # No "$and" means no refinement selected
    if "$and" not in query:
        return {}

    for element_or in query["$and"]:
        if "$or" not in element_or.keys():
            continue

        # Go through all '$or' => where refinement are
        for element in element_or["$or"]:
            for key, value in list(element.items()):
                # Do not parse path ending with "#text" and parse only "$in"
                if key.endswith("#text") or "$in" not in value:
                    continue

                for selected_value in value["$in"]:
                    # Do not parse categories ending with '__category'
                    if selected_value.endswith("__category"):
                        continue

                    # Build category value list
                    if key in category_values_list:
                        category_values_list[key].append(selected_value)
                    else:
                        category_values_list.update({key: [selected_value]})

    # get global template.
    template = template_registry_api.get_current_registry_template(
        request=request
    )
    # get refinements.
    refinements = refinement_api.get_all_filtered_by_template_hash(
        template.hash
    )
    refinements_ids = [x.id for x in refinements]
    # get all category.
    q_list = []
    for key, values in list(category_values_list.items()):
        # prepare the query
        q_list.append(
            Q(path=key)
            & Q(refinement_id__in=refinements_ids)
            & Q(value__in=values)
        )

    if len(q_list) == 0:  # No refinement found
        return return_value

    # if refinement are found, we build the structure
    categories = category_api.get_all().filter((reduce(operator.or_, q_list)))
    # now we have to build a list of {refinement name: category ids, }
    for category in categories:
        key = category.refinement.slug
        display_name = category.refinement.name
        if key in return_value:
            return_value[key][display_name].append(
                {"id": category.id, "value": category.value.split(":")[0]}
            )
        else:
            return_value.update(
                {
                    key: {
                        display_name: [
                            {
                                "id": category.id,
                                "value": category.value.split(":")[0],
                            }
                        ]
                    }
                }
            )
    # return the structure
    return return_value


def add_not_deleted_status_criteria():
    """Adds a criteria on status. Status should not be deleted.

    Returns:
        Criteria

    """
    return {PATH_STATUS: {"$ne": DataStatus.DELETED}}
